@@ -53,6 +53,7 @@
 #include <fnmatch.h>
 #include <sys/select.h>
 #include <grp.h>
+#include <time.h>
 
 #undef PACKAGE
 #undef PACKAGE_BUGREPORT
@@ -73,12 +74,13 @@ typedef struct Daisy
 {
    int playorder, x, y, screen;
    float begin, duration;
-   char *xml_file, *anchor, *clips_file, *clips_anchor, *orig_smil;
+   char *xml_file, *anchor, *clips_file;
+   char *clips_anchor, *orig_smil;
    char class[MAX_STR], label[100];
-   char first_id[MAX_STR + 1], last_id[MAX_STR + 1];
+   char first_id[MAX_STR], last_id[MAX_STR];
    int level, page_number;
-   char daisy_mp[MAX_STR]; // discinfo
-   char filename[MAX_STR]; // Audio-CD
+   char *daisy_mp; // discinfo
+   char *filename; // Audio-CD
    lsn_t first_lsn, last_lsn;
 } daisy_t;
 
@@ -103,8 +105,8 @@ typedef struct My_attribute
         ncc_totalTime[MAX_STR],
         number[MAX_STR],
         playorder[MAX_STR],
-        *src,
         smilref[MAX_STR],
+        *src,
         toc[MAX_STR],
         value[MAX_STR];
 } my_attribute_t;
@@ -117,7 +119,8 @@ typedef struct Misc
    int tts_no, depth, total_pages;
    int pipefd[2], tmp_wav_fd, has_audio_tag;
    int pause_resume_playing, mounted_by_daisy_player;
-   char *pause_resume_id, *prev_id, *current_id, *audio_id;
+   char *pause_resume_id, *prev_id, *current_id;
+   char *audio_id;
    float speed, total_time, clip_begin, clip_end;
    long min_vol, max_vol, volume;
    htmlDocPtr doc;
@@ -129,14 +132,13 @@ typedef struct Misc
    char *daisy_mp, *tmp_dir;
    char tag[MAX_TAG], *label;
    int label_len;
-   char bookmark_title[MAX_STR], *search_str;
-   char cd_dev[MAX_STR], *pulseaudio_device;
+   char bookmark_title[MAX_STR], search_str[30];
+   char cd_dev[MAX_STR], pulseaudio_device[5];
    char cddb_flag, opf_name[MAX_STR], ncx_name[MAX_STR];
    char use_ncx, use_opf;
    char *current_audio_file, tmp_wav[MAX_STR + 1], mcn[MAX_STR];
    char xmlversion[MAX_STR + 1];
    char cmd[MAX_CMD + 1], str[MAX_STR + 1];
-   char *case_insensitive_file_name;
    time_t elapsed_seconds;
    WINDOW *screenwin, *titlewin;
    cdrom_paranoia_t *par;
@@ -147,53 +149,78 @@ typedef struct Misc
    int use_OPF, use_NCX; // for testing
 } misc_t;
 
-extern void save_xml (misc_t *);
-extern char *get_mcn (misc_t *);
-extern void player_ended ();
-extern int get_tag_or_label (misc_t *, my_attribute_t *,
-                                    xmlTextReaderPtr);
-extern void get_label_3 (misc_t *, my_attribute_t *, daisy_t *, int,
-                       xmlTextReaderPtr);
-extern void parse_ncx (misc_t *, my_attribute_t *, daisy_t *);
-extern void skip_left (misc_t *, my_attribute_t *, daisy_t *);
-extern void skip_right (misc_t *, daisy_t *);
-extern void usage ();
-extern pid_t play_track (misc_t *, char *, char *, lsn_t);
-extern void get_clips (misc_t *, my_attribute_t *);
-extern void open_clips_file (misc_t *, my_attribute_t *, char *, char *);
-extern void open_text_file (misc_t *, my_attribute_t *, char *, char *);
-extern void play_now (misc_t *, daisy_t *);
-extern void start_playing (misc_t *, daisy_t *);
-extern int get_page_number_2 (misc_t *, my_attribute_t *, daisy_t *, char *);
-extern int get_page_number_3 (misc_t *, my_attribute_t *);
-extern void get_next_clips (misc_t *, my_attribute_t *, daisy_t *);
-extern void set_drive_speed (misc_t *, int);
-extern void quit_daisy_player (misc_t *, daisy_t *);
-extern void read_daisy_3 (misc_t *, my_attribute_t *, daisy_t *);
-extern void pause_resume (misc_t *, my_attribute_t *, daisy_t *);
-extern float read_time (char *);  
-extern void init_paranoia (misc_t *);
 extern void get_toc_audiocd (misc_t *, daisy_t *);
-extern void playfile (misc_t *, char *, char *, char *, char *, char *);
+extern  daisy_t *get_number_of_tracks (misc_t *);
+extern pid_t play_track (misc_t *, char *, char *, lsn_t);
+extern void playfile (char *, char *, char *, char *, char *);
+extern char *get_mcn (misc_t *);
+extern void quit_daisy_player (misc_t *, my_attribute_t *, daisy_t *);
 extern void view_screen (misc_t *, daisy_t *);
-extern daisy_t *create_daisy_struct (misc_t *, my_attribute_t *, daisy_t *);
-extern daisy_t *get_number_of_tracks (misc_t *);
-extern void failure (misc_t *, char *, int);
-extern void put_bookmark (misc_t *);
-extern void parse_page_number (misc_t *, my_attribute_t *, xmlTextReaderPtr);
-extern void fill_daisy_struct_2 (misc_t *, my_attribute_t *, daisy_t *);
-extern void fill_page_numbers (misc_t *, daisy_t *, my_attribute_t *);
-extern char *convert_URL_name (misc_t *, char *);
-extern void go_to_page_number (misc_t *, my_attribute_t *, daisy_t *);
-extern void parse_smil_3 (misc_t *, my_attribute_t *, daisy_t *);
-extern void remove_tmp_dir (misc_t *);
-extern void make_tmp_dir (misc_t *);
-extern char *find_index_name (misc_t *, char *);
+extern void pause_resume (misc_t *, my_attribute_t *, daisy_t *);
+extern void open_clips_file (misc_t *, my_attribute_t *, char *, char *);
+extern void get_next_clips (misc_t *, my_attribute_t *, daisy_t *);
+extern void free_all (misc_t *, my_attribute_t *, daisy_t *);
 extern void select_next_output_device (misc_t *, daisy_t *);
-extern void get_volume (misc_t *);
-extern void set_volume (misc_t *);
+extern void go_to_page_number (misc_t *, my_attribute_t *, daisy_t *);
+extern void make_tmp_dir (misc_t *);
+extern daisy_t *create_daisy_struct (misc_t *, my_attribute_t *, daisy_t *);
+extern void skip_right (misc_t *, daisy_t *);
+extern void player_ended ();
+extern char *convert_URL_name (misc_t *, char *);
+extern void get_path_name (char *, char *, char *);
+extern void failure (misc_t *, char *, int);
+extern int get_tag_or_label (misc_t *, my_attribute_t *, xmlTextReaderPtr);
+extern void get_clips (misc_t *, my_attribute_t *);
+extern void fill_daisy_struct_2 (misc_t *, my_attribute_t *, daisy_t *);
+extern void fill_item (misc_t *, daisy_t *);
+extern int get_page_number_2 (misc_t *, my_attribute_t *,
+                              daisy_t *daisy, char *);
+extern void read_daisy_3 (misc_t *, my_attribute_t *, daisy_t *);
+extern void fill_page_numbers (misc_t *, daisy_t *, my_attribute_t *);
+extern int get_page_number_3 (misc_t *, my_attribute_t *);
 extern int madplay (char *, char *, char *, char *);
 extern void kill_player (misc_t *);
-extern void get_path_name (char *, char *, char *);
-extern long time (char *);
-extern void pactl (char *, char *);
+extern void remove_tmp_dir (misc_t *);
+extern void init_paranoia (misc_t *);
+extern void parse_page_number (misc_t *, my_attribute_t *, xmlTextReaderPtr);
+extern char *get_dir_content (misc_t *, char *, char *);
+extern void get_cddb_info (misc_t *, daisy_t *);
+extern void find_index_names (misc_t *);
+extern int handle_ncc_html (misc_t *, my_attribute_t *, daisy_t *);
+extern int namefilter (const struct dirent *);
+extern int get_meta_attributes (xmlTextReaderPtr, xmlTextWriterPtr);
+extern void create_ncc_html (misc_t *);
+extern void get_attributes (misc_t *, my_attribute_t *, xmlTextReaderPtr);
+extern char *pactl (char *, char *, char *);
+extern void parse_smil_2 (misc_t *, my_attribute_t *, daisy_t *);
+extern void get_label_2 (misc_t *, daisy_t *, int);
+extern void fill_xml_anchor_ncx (misc_t *, my_attribute_t *, daisy_t *);
+extern void parse_content (misc_t *, my_attribute_t *, daisy_t *);
+extern void parse_text (misc_t *, my_attribute_t *, daisy_t *);
+extern void put_bookmark (misc_t *);
+extern float read_time (char *);
+extern void parse_clips (misc_t *, my_attribute_t *, daisy_t *);
+extern void parse_smil_3 (misc_t *, my_attribute_t *, daisy_t *);
+extern void fill_xml_anchor_opf (misc_t *, my_attribute_t *, daisy_t *);
+extern void parse_opf (misc_t *, my_attribute_t *, daisy_t *);
+extern void parse_ncx (misc_t *, my_attribute_t *, daisy_t *);
+extern void get_bookmark (misc_t *, my_attribute_t *, daisy_t *);
+extern void view_page (misc_t *, daisy_t *);
+extern void view_time (misc_t *, daisy_t *);
+extern void start_playing (misc_t *, daisy_t *);
+extern void write_wav (misc_t *, my_attribute_t *, daisy_t *, char *);
+extern void store_to_disk (misc_t *, my_attribute_t *, daisy_t *);
+extern void help (misc_t *, my_attribute_t *, daisy_t *);
+extern void previous_item (misc_t *, daisy_t *);
+extern void next_item (misc_t *, daisy_t *);
+extern void calculate_times_3 (misc_t *, my_attribute_t *, daisy_t *);
+extern void load_xml (misc_t *, my_attribute_t *);
+extern void save_xml (misc_t *);
+extern void search (misc_t *, my_attribute_t *, daisy_t *, int, char);
+extern void change_level (misc_t *, my_attribute_t *, daisy_t *, char);
+extern void go_to_time (misc_t *, daisy_t *, my_attribute_t *);
+extern void skip_left (misc_t *, my_attribute_t *, daisy_t *);
+extern void browse (misc_t *, my_attribute_t *, daisy_t *, char *);
+extern void usage (int);
+extern char *get_mount_point (misc_t *);
+extern void handle_discinfo (misc_t *, my_attribute_t *, daisy_t *, char *);
